/**
 * @Arduino Ian FIFO  controller
 *
 * Copyright (C) Ian Jin 
 * iancanada.mail@gmail.com
 * Twitter: @iancanadaTT
 * 
 * 2015-08-30 First release Ver1.00
 * 
 * Tested on Arduino Uno and MEGA 2560
 * need a DRF0009 LCD KeyPad shield
 * works with:
 * 1. FIFO II with/without a S/PDIF board
 * 2. FIFO I with a S/PDIF board
 * 
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

  /**
  * conect to Ian FIFO KIT
  * DFR0009 PINS    Ian FIFO KIT PINS       SIGNAL NAME  INPUT/OUTPUT      NOTE
  *     D0 ---------- JUMPER1 (FIFO II)       fifoPin      OUTPUT          not connected for FIFO I
  *     D1 ---------- JUMPER2 (FIFO II)       i2s1Pin      INPUT PULLUP    not connected for FIFO I
  *     D2 ---------- JUMPER3 (FIFO II)       i2s2Pin      INPUT PULLUP    not connected for FIFO I
  *     D3 ---------- K1+     (S/PDIF board)  spdifPin     OUTPUT          not connected if there is no S/PDIF board
  *     D11 --------- COX+    (S/PDIF board)  coxPin       INPUT PULLUP    not connected if there is no S/PDIF board
  *     D12 --------- OPT+    (S/PDIF board)  optPin       INPUT PULLUP    not connected if there is no S/PDIF board 
  *     D13 --------- TTL+    (S/PDIF board)  ttlPin       INPUT PULLUP    not connected if there is no S/PDIF board     
  *     
  *     GND --------- JUMPER1GND(FIFO II) or COX-(S/PDIF board)        has to be connected in either case      
  *     
  *     see IanFIFO.h for more details
  */
 
#include "IanArduinoRTOS.h"
#include "IanFIFOII.h"
#include "DFR0009.h"
#include <LiquidCrystal.h>

CIanFIFO IanFIFO(0,1,2,3,11,12,13);  //initial IanFIFO class
DFR0009 Key(0);                      //analog pin 0 for DFR0009 key pin
LiquidCrystal lcd(8, 9, 4, 5, 6, 7); //select pins used on the LCD panel of DFR0009
int flashTime=0;                     //time(in 10ms) to flash slection display after key pressed
int noSpdifFlag=true;                //if ture, indicate there is no spdif board by defult. Will be clear later if vaild spdif sources is updated 

//Custom Character Generator
//http://omerk.github.io/lcdchargen/
byte musicChar1[8] = {
  0b11111,
  0b11111,
  0b11001,
  0b11011,
  0b11011,
  0b10011,
  0b10011,
  0b11111
};
byte musicChar2[8] = {
0b11111,
  0b11001,
  0b11011,
  0b11011,
  0b10011,
  0b10011,
  0b11111,
  0b11111
};
byte fChar[8] = {
  0b00000,
  0b11111,
  0b10001,
  0b10111,
  0b10011,
  0b10111,
  0b10111,
  0b11111
};
byte iChar[8] = {
  0b00000,
  0b11111,
  0b11011,
  0b11011,
  0b11011,
  0b11011,
  0b11011,
  0b11111
};
byte oChar[8] = {
  0b00000,
  0b11111,
  0b10001,
  0b10101,
  0b10101,
  0b10101,
  0b10001,
  0b11111
};


//define LCD display macros
#define _showI2S1selected lcd.setCursor(0,1);lcd.write(1);lcd.write(2);lcd.write(1);lcd.write(2)
#define _showI2S2selected lcd.setCursor(5,1);lcd.write(1);lcd.write(2);lcd.write(1);if(noSpdifFlag) lcd.write(2)
#define _clearShownSelection lcd.setCursor(0,1);lcd.print("         ")
#define _showFIFO lcd.setCursor(12,1);lcd.write(3);lcd.write(4);lcd.write(3);lcd.write(5)

void setup() 
{                 
  lcd.begin(16, 2);                                                               //start LCD
  delay(800);                                                                     //800ms wait for FIFO power up
  lcd.createChar(1, musicChar1);  lcd.createChar(2, musicChar2);                  //register custom characters 1,2
  lcd.createChar(3, fChar);  lcd.createChar(4, iChar);  lcd.createChar(5, oChar); //register custom characters 3,4,5

  _showFIFO;                              //show "FIFO" at right bottom corner
  lcd.setCursor(5,0);lcd.print("I2S2");   //show "I2S2" as default in case there is no spdif board
  _showI2S2selected;                      //as default just in case FIFO I is in use
}

void loop() 
{  
  IanOS.run();                    //run Ian Arduino RTOS  
  
  //processes driven by events
  
  if(IanFIFO.checkFifoFlag())     //FIFO source changed event
  {
    _clearShownSelection;         //clear all possible selection display in advance
    if(IanFIFO.getFifoSource()==FIFOI2S1)
    {
      lcd.setCursor(0,0);lcd.print("I2S1");  //to display "I2S1" because default was empty in case FIFO I is in use
    }
    refreshSelectionDisplay();
  }

  if(IanFIFO.checkSpdifFlag())    //spdif board source changed event
  {
    lcd.setCursor(5,0);
    switch(IanFIFO.getSpdifSource())
    {
      case OPT:                   //opt is selected
        noSpdifFlag=0;            //clear the default noSpdifFlag since spdif opt signal is there
        lcd.print("OPT BNC TTL");
        break;
      case COX:                   //bnc is selected
        noSpdifFlag=0;            //clear the default noSpdifFlag since spdif cox signal is there
        lcd.print("BNC TTL BKD");
        break;
      case TTL:                   //ttl is disconnected 
        noSpdifFlag=0;            //clear the default noSpdifFlag since spdif ttl signal is there
        lcd.print("TTL BKD OPT");
        break;      
      case BKD:                   //i2s backdoor is selected  
        lcd.print("BKD OPT BNC");
        break;
    }
  }  
  
  switch(Key.getflag())            //possible key pressed event
  {
    case btnRIGHT:                 //rignt button pressed
      refreshSelectionDisplay();   //refresh selection display in case flashTims was launched and at the time of icon was cleared 
      flashTime=70;                //launch total flash time 700ms
      if(IanFIFO.getFifoPin()!=0)  //current fifo pin is 1 which means I2S1 is selected
        IanFIFO.setFifoPin(0);     //to slect I2S2 in this case
      else                         //already I2S2, need to press spdif button
        IanFIFO.launchSpdifPin(30);//press spdif button for 300ms         
      break;
    case btnLEFT:                  //left button pressed
    case btnSELECT:                //or select button pressed
      refreshSelectionDisplay();   //refresh selection display in case flashTims was launched and at the time of icon was cleared 
      flashTime=70;                //launch total flash time 700ms
      if(IanFIFO.getFifoPin()==0)  //current fofo pin is 10which means I2S2 is selected
        IanFIFO.setFifoPin(1);     //to slect I2S2 in this case
      break;      
  }  
}

//IanOS scheduled tasks
void IanOStasks(void)  //by default, run once every 10 ms in the background
{
  //scheduled task 1, to scan changes on fifo/spdif sources
  IanFIFO.scan();
  
  //scheduled task 2, to scan key activities of DFR0009
  Key.scan();
  
  //schedule task3, to flash slection display if flashTime is set
  if(flashTime!=0)
  {
    flashTime--;
    if(flashTime==30)               //300ms before time up
    {
      _clearShownSelection;         //clear all possible selection display
    }
    if(flashTime==0)                //when time is up, flash seletion display back to normal
      refreshSelectionDisplay();
  }
  //all scheculed tasks are finshed
}

//function of refreshing  selection(I2S1 or I2S2/SPDIF) display on LCD according to current FIFO source status
void refreshSelectionDisplay()
{
  switch(IanFIFO.getFifoSource())
  { 
    case FIFOI2S1:              //I2S1 is selected
      _showI2S1selected;
      break;        
    case FIFOI2S2:              //I2S2 is selected
    case FIFOIIDSCONNECT:       //or FIFO I is in use
      _showI2S2selected;
      break;        
  }              
}

